@@ -40,6 +40,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.ResourceBundle;
+import java.util.concurrent.Executor;
 import java.util.concurrent.Executors;
 import java.util.prefs.BackingStoreException;
 import java.util.prefs.Preferences;
@@ -97,7 +98,7 @@ import com.eteks.sweethome3d.viewcontroller.ViewFactory;
  * one of its <code>furniture</code>, <code>textures</code>, <code>languages</code>, <code>plugins</code>,
  * <code>recovery</code> subfolders don't exist, Sweet Home 3D will create it when needed.<br>
  * The other folders are used as resources where SH3F, SH3T, SH3L, SH3P files will be searched 
- * in their respective code>furniture</code>, <code>textures</code>, <code>languages</code>, <code>plugins</code> 
+ * in their respective <code>furniture</code>, <code>textures</code>, <code>languages</code>, <code>plugins</code> 
  * subfolders. Any of the cited folders may be an absolute path or a relative path to the folder
  * from which the program was launched.</li>
  *       
@@ -197,7 +198,12 @@ public class SweetHome3D extends HomeApplication {
       } else {
         applicationFolders = null;
       }
-      this.userPreferences = new FileUserPreferences(preferencesFolder, applicationFolders);
+      this.userPreferences = new FileUserPreferences(preferencesFolder, applicationFolders, 
+          new Executor() {
+            public void execute(Runnable command) {
+              EventQueue.invokeLater(command);
+            }
+          });
     }
     return this.userPreferences;
   }
@@ -207,7 +213,7 @@ public class SweetHome3D extends HomeApplication {
    */
   protected ContentManager getContentManager() {
     if (this.contentManager == null) {
-      this.contentManager = new FileContentManager(getUserPreferences());
+      this.contentManager = new FileContentManagerWithRecordedLastDirectories(getUserPreferences(), getClass());
     }
     return this.contentManager;
   }
@@ -599,7 +605,9 @@ public class SweetHome3D extends HomeApplication {
             public void collectionChanged(CollectionEvent<Home> ev) {
               if (ev.getType() == CollectionEvent.Type.ADD) {
                 removeHomesListener(this);
-                autoRecoveryManager.openRecoveredHomes();                
+                if (autoRecoveryManager != null) {
+                  autoRecoveryManager.openRecoveredHomes();
+                }
               }
             }
           });
@@ -654,7 +662,9 @@ public class SweetHome3D extends HomeApplication {
         });
       }
     } else if (getHomes().isEmpty()) {
-      this.autoRecoveryManager.openRecoveredHomes();
+      if (autoRecoveryManager != null) {
+        this.autoRecoveryManager.openRecoveredHomes();
+      }
       if (getHomes().isEmpty()) {
         // Add a new home to application
         addHome(createHome());
@@ -685,6 +695,65 @@ public class SweetHome3D extends HomeApplication {
     }
   }
 
+  /**
+   * A file content manager that records the last directories for each content
+   * in Java preferences.
+   */
+  private static class FileContentManagerWithRecordedLastDirectories extends FileContentManager {
+    private static final String LAST_DIRECTORY         = "lastDirectory#";
+    private static final String LAST_DEFAULT_DIRECTORY = "lastDefaultDirectory";
+    
+    private final Class<? extends SweetHome3D> mainClass;
+
+    public FileContentManagerWithRecordedLastDirectories(UserPreferences preferences, 
+                                                         Class<? extends SweetHome3D> mainClass) {
+      super(preferences);
+      this.mainClass = mainClass;
+    }
+
+    @Override
+    protected File getLastDirectory(ContentType contentType) {
+      Preferences preferences = Preferences.userNodeForPackage(this.mainClass);
+      String directoryPath = null;
+      if (contentType != null) {
+        directoryPath = preferences.get(LAST_DIRECTORY + contentType, null);
+      }
+      if (directoryPath == null) {
+        directoryPath = preferences.get(LAST_DEFAULT_DIRECTORY, null);
+      }
+      if (directoryPath != null) {
+        File directory = new File(directoryPath);
+        if (directory.isDirectory()) {
+          return directory;
+        } 
+      }
+      return null;
+    }
+    
+    @Override
+    protected void setLastDirectory(ContentType contentType, File directory) {
+      // Last directories are not recorded in user preferences since there's no need of portability 
+      // from a computer to an other
+      Preferences preferences = Preferences.userNodeForPackage(this.mainClass);
+      if (directory == null) {
+        preferences.remove(LAST_DIRECTORY + contentType);
+      } else {
+        String directoryPath = directory.getAbsolutePath();
+        if (contentType != null) {
+          preferences.put(LAST_DIRECTORY + contentType, directoryPath);
+        }
+        if (directoryPath != null) {
+          preferences.put(LAST_DEFAULT_DIRECTORY, directoryPath);
+        }
+      }
+      try {
+        preferences.flush();
+      } catch (BackingStoreException ex) {
+        // Ignore exception, Sweet Home 3D will work without recorded directories
+      }
+    }
+  }
+  
   /**
    * JNLP <code>ServiceManagerStub</code> implementation for standalone
    * applications run out of Java Web Start. This service manager supports
@@ -692,9 +761,9 @@ public class SweetHome3D extends HomeApplication {
    * .
    */
   private static class StandaloneServiceManager implements ServiceManagerStub {
-    private final Class<?> mainClass;
+    private final Class<? extends SweetHome3D> mainClass;
 
-    public StandaloneServiceManager(Class<?> mainClass) {
+    public StandaloneServiceManager(Class<? extends SweetHome3D> mainClass) {
       this.mainClass = mainClass;
     }
 
@@ -796,10 +865,10 @@ public class SweetHome3D extends HomeApplication {
   private static class StandaloneSingleInstanceService implements SingleInstanceService {
     private static final String                SINGLE_INSTANCE_PORT    = "singleInstancePort";
 
-    private final Class<?>                     mainClass;
+    private final Class<? extends SweetHome3D> mainClass;
     private final List<SingleInstanceListener> singleInstanceListeners = new ArrayList<SingleInstanceListener>();
 
-    public StandaloneSingleInstanceService(Class<?> mainClass) {
+    public StandaloneSingleInstanceService(Class<? extends SweetHome3D> mainClass) {
       this.mainClass = mainClass;
     }
 
@@ -880,7 +949,7 @@ public class SweetHome3D extends HomeApplication {
      * Returns <code>true</code> if single instance server was successfully
      * called.
      */
-    public static boolean callSingleInstanceServer(String [] mainArgs, Class<?> mainClass) {
+    public static boolean callSingleInstanceServer(String [] mainArgs, Class<? extends SweetHome3D> mainClass) {
       if (!OperatingSystem.isMacOSX()) {
         // No server under Mac OS X, multiple application launches are managed
         // by com.apple.eawt.ApplicationListener in MacOSXConfiguration class
