@@ -46,6 +46,8 @@ import java.util.Map.Entry;
 import java.util.Properties;
 import java.util.Set;
 import java.util.TreeSet;
+import java.util.concurrent.Executor;
+import java.util.concurrent.Executors;
 import java.util.prefs.AbstractPreferences;
 import java.util.prefs.BackingStoreException;
 import java.util.prefs.Preferences;
@@ -58,7 +60,6 @@ import com.eteks.sweethome3d.model.CatalogTexture;
 import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.FurnitureCategory;
-import com.eteks.sweethome3d.model.IllegalHomonymException;
 import com.eteks.sweethome3d.model.LengthUnit;
 import com.eteks.sweethome3d.model.PatternsCatalog;
 import com.eteks.sweethome3d.model.RecorderException;
@@ -133,6 +134,8 @@ public class FileUserPreferences extends UserPreferences {
   private final File                 preferencesFolder;
   private final File []              applicationFolders;
   private Preferences                preferences;
+  private Executor                   catalogsLoader;
+  private Executor                   updater;
   
   static {
     Content dummyURLContent = null;
@@ -161,13 +164,43 @@ public class FileUserPreferences extends UserPreferences {
    *    have write access otherwise the user won't be able to import them.
    */
   public FileUserPreferences(File preferencesFolder,
-                             File [] applicationFolders) {    
+                             File [] applicationFolders) {
+    this(preferencesFolder, applicationFolders, null);
+  }
+  
+  /**
+   * Creates user preferences stored in the folders given in parameter. 
+   * @param preferencesFolder the folder where preferences files are stored
+   *    or <code>null</code> if this folder is the default one.
+   * @param applicationFolders  the folders where application private files are stored
+   *    or <code>null</code> if it's the default one. As the first application folder
+   *    is used as the folder where plug-ins files are imported by the user, it should
+   *    have write access otherwise the user won't be able to import them.
+   * @param updater  an executor that will be used to update user preferences for lengthy 
+   *    operations. If <code>null</code>, then these operations and 
+   *    updates will be executed in the current thread.
+   */
+  public FileUserPreferences(File preferencesFolder,
+                             File [] applicationFolders,
+                             Executor updater) {
     this.preferencesFolder = preferencesFolder;
     this.applicationFolders = applicationFolders;
+    Executor defaultExecutor = new Executor() {
+        public void execute(Runnable command) {
+          command.run();
+        }
+      };
+    if (updater == null) {
+      this.catalogsLoader =
+      this.updater = defaultExecutor;
+    } else {
+      this.catalogsLoader = Executors.newSingleThreadExecutor();
+      this.updater = updater;
+    }
     
     updateSupportedLanguages();
 
-    Preferences preferences;
+    final Preferences preferences;
     // From version 3.0 use portable preferences
     PortablePreferences portablePreferences = new PortablePreferences();    
     // If portable preferences storage doesn't exist and default preferences folder is used
@@ -185,17 +218,19 @@ public class FileUserPreferences extends UserPreferences {
       language = Locale.ENGLISH.getLanguage();  
     }
     setLanguage(language);    
-    
-    // Fill default furniture catalog 
-    setFurnitureCatalog(new DefaultFurnitureCatalog(this, getFurnitureLibrariesPluginFolders()));
-    // Read additional furniture
-    readFurnitureCatalog(preferences);
-    
-    // Fill default textures catalog 
-    setTexturesCatalog(new DefaultTexturesCatalog(this, getTexturesLibrariesPluginFolders()));
-    // Read additional textures
-    readTexturesCatalog(preferences);
 
+    setFurnitureCatalog(new FurnitureCatalog());
+    // Fill default furniture catalog 
+    updateFurnitureDefaultCatalog(defaultExecutor, defaultExecutor);
+    // Read additional furniture
+    readModifiableFurnitureCatalog(preferences);
+    
+    setTexturesCatalog(new TexturesCatalog());
+    // Fill default textures catalog 
+    updateTexturesDefaultCatalog(defaultExecutor, defaultExecutor);
+    // Read additional textures
+    readModifiableTexturesCatalog(preferences);
+    
     DefaultUserPreferences defaultPreferences = new DefaultUserPreferences(false, this);
     
     // Fill default patterns catalog 
@@ -269,17 +304,19 @@ public class FileUserPreferences extends UserPreferences {
     
     addPropertyChangeListener(Property.LANGUAGE, new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent ev) {
-          updateFurnitureDefaultCatalog();
-          updateTexturesDefaultCatalog();
+          // Update catalogs with new default locale
+          updateFurnitureDefaultCatalog(catalogsLoader, FileUserPreferences.this.updater);
+          updateTexturesDefaultCatalog(catalogsLoader, FileUserPreferences.this.updater);
           updateAutoCompletionStrings();
         }
       });
     
     if (preferences != portablePreferences) {
       // Switch to portable preferences now that all preferences are read
-      preferences = portablePreferences;
+      this.preferences = portablePreferences;
+    } else {
+      this.preferences = preferences;
     }
-    this.preferences = preferences;
   }
   
   /**
@@ -374,9 +411,10 @@ public class FileUserPreferences extends UserPreferences {
   /**
    * Reloads furniture default catalogs.
    */
-  private void updateFurnitureDefaultCatalog() {
+  private void updateFurnitureDefaultCatalog(Executor furnitureCatalogLoader, 
+                                             final Executor updater) {
     // Delete default pieces of current furniture catalog          
-    FurnitureCatalog furnitureCatalog = getFurnitureCatalog();
+    final FurnitureCatalog furnitureCatalog = getFurnitureCatalog();
     for (FurnitureCategory category : furnitureCatalog.getCategories()) {
       for (CatalogPieceOfFurniture piece : category.getFurniture()) {
         if (!piece.isModifiable()) {
@@ -384,27 +422,32 @@ public class FileUserPreferences extends UserPreferences {
         }
       }
     }
-    // Read again default furniture catalog with new default locale
-    // Add default pieces that don't have homonym among user catalog
-    FurnitureCatalog defaultFurnitureCatalog = 
-        new DefaultFurnitureCatalog(this, getFurnitureLibrariesPluginFolders());
-    for (FurnitureCategory category : defaultFurnitureCatalog.getCategories()) {
-      for (CatalogPieceOfFurniture piece : category.getFurniture()) {
-        try {
-          furnitureCatalog.add(category, piece);
-        } catch (IllegalHomonymException ex) {
-          // Ignore pieces that have the same name as an existing piece
+    // Read default furniture catalog
+    furnitureCatalogLoader.execute(new Runnable() {
+        public void run() {
+          // Fill default furniture catalog 
+          DefaultFurnitureCatalog defaultFurnitureCatalog = new DefaultFurnitureCatalog(
+              FileUserPreferences.this, getFurnitureLibrariesPluginFolders());
+          for (final FurnitureCategory category : defaultFurnitureCatalog.getCategories()) {
+            for (final CatalogPieceOfFurniture piece : category.getFurniture()) {
+              updater.execute(new Runnable() {
+                  public void run() {
+                    furnitureCatalog.add(category, piece);
+                  }
+                });
+            }
+          }
         }
-      }
-    }
+      });
   }
 
   /**
    * Reloads textures default catalog.
    */
-  private void updateTexturesDefaultCatalog() {
+  private void updateTexturesDefaultCatalog(Executor texturesCatalogLoader, 
+                                            final Executor updater) {
     // Delete default textures of current textures catalog          
-    TexturesCatalog texturesCatalog = getTexturesCatalog();
+    final TexturesCatalog texturesCatalog = getTexturesCatalog();
     for (TexturesCategory category : texturesCatalog.getCategories()) {
       for (CatalogTexture texture : category.getTextures()) {
         if (!texture.isModifiable()) {
@@ -412,19 +455,22 @@ public class FileUserPreferences extends UserPreferences {
         }
       }
     }
-    // Read again default textures catalog with new default locale
-    // Add default textures that don't have homonym among user catalog
-    TexturesCatalog defaultTexturesCatalog = 
-        new DefaultTexturesCatalog(this, getTexturesLibrariesPluginFolders());
-    for (TexturesCategory category : defaultTexturesCatalog.getCategories()) {
-      for (CatalogTexture texture : category.getTextures()) {
-        try {
-          texturesCatalog.add(category, texture);
-        } catch (IllegalHomonymException ex) {
-          // Ignore textures that have the same name as an existing piece
+    // Read default textures catalog
+    texturesCatalogLoader.execute(new Runnable() {
+        public void run() {
+          TexturesCatalog defaultTexturesCatalog = new DefaultTexturesCatalog(
+              FileUserPreferences.this, getTexturesLibrariesPluginFolders());
+          for (final TexturesCategory category : defaultTexturesCatalog.getCategories()) {
+            for (final CatalogTexture texture : category.getTextures()) {
+              updater.execute(new Runnable() {
+                  public void run() {
+                    texturesCatalog.add(category, texture);
+                  }
+                });
+            }
+          }
         }
-      }
-    }
+      });
   }
 
   /**
@@ -440,9 +486,9 @@ public class FileUserPreferences extends UserPreferences {
   }
 
   /**
-   * Read furniture catalog from preferences.
+   * Read modifiable furniture catalog from preferences.
    */
-  private void readFurnitureCatalog(Preferences preferences) {
+  private void readModifiableFurnitureCatalog(Preferences preferences) {
     for (int i = 1; ; i++) {
       String name = preferences.get(FURNITURE_NAME + i, null);
       if (name == null) {
@@ -467,8 +513,8 @@ public class FileUserPreferences extends UserPreferences {
       float iconYaw = preferences.getFloat(FURNITURE_ICON_YAW + i, 0);
       boolean proportional = preferences.getBoolean(FURNITURE_PROPORTIONAL + i, true);
 
-      FurnitureCategory pieceCategory = new FurnitureCategory(category);
-      CatalogPieceOfFurniture piece;
+      final FurnitureCategory pieceCategory = new FurnitureCategory(category);
+      final CatalogPieceOfFurniture piece;
       if (doorOrWindow) {
         piece = new CatalogDoorOrWindow(name, icon, model,
             width, depth, height, elevation, movable, 1, 0, new Sash [0],
@@ -478,19 +524,7 @@ public class FileUserPreferences extends UserPreferences {
             width, depth, height, elevation, movable, 
             staircaseCutOutShape, color, modelRotation, backFaceShown, iconYaw, proportional);
       }
-      try {        
-        getFurnitureCatalog().add(pieceCategory, piece);
-      } catch (IllegalHomonymException ex) {
-        // If a piece with same name and category already exists in furniture catalog
-        // replace the existing piece by the new one
-        List<FurnitureCategory> categories = getFurnitureCatalog().getCategories();
-        int categoryIndex = Collections.binarySearch(categories, pieceCategory);
-        List<CatalogPieceOfFurniture> furniture = categories.get(categoryIndex).getFurniture();
-        int existingPieceIndex = Collections.binarySearch(furniture, piece);        
-        getFurnitureCatalog().delete(furniture.get(existingPieceIndex));
-        
-        getFurnitureCatalog().add(pieceCategory, piece);
-      }
+      getFurnitureCatalog().add(pieceCategory, piece);
     }
   }  
 
@@ -545,9 +579,9 @@ public class FileUserPreferences extends UserPreferences {
   }
   
   /**
-   * Read textures catalog from preferences.
+   * Read modifiable textures catalog from preferences.
    */
-  private void readTexturesCatalog(Preferences preferences) {
+  private void readModifiableTexturesCatalog(Preferences preferences) {
     for (int i = 1; ; i++) {
       String name = preferences.get(TEXTURE_NAME + i, null);
       if (name == null) {
@@ -559,21 +593,9 @@ public class FileUserPreferences extends UserPreferences {
       float width = preferences.getFloat(TEXTURE_WIDTH + i, 0.1f);
       float height = preferences.getFloat(TEXTURE_HEIGHT + i, 0.1f);
 
-      TexturesCategory textureCategory = new TexturesCategory(category);
-      CatalogTexture texture = new CatalogTexture(name, image, width, height, true);
-      try {        
-        getTexturesCatalog().add(textureCategory, texture);
-      } catch (IllegalHomonymException ex) {
-        // If a texture with same name and category already exists in textures catalog
-        // replace the existing texture by the new one
-        List<TexturesCategory> categories = getTexturesCatalog().getCategories();
-        int categoryIndex = Collections.binarySearch(categories, textureCategory);
-        List<CatalogTexture> textures = categories.get(categoryIndex).getTextures();
-        int existingTextureIndex = Collections.binarySearch(textures, texture);        
-        getTexturesCatalog().delete(textures.get(existingTextureIndex));
-        
-        getTexturesCatalog().add(textureCategory, texture);
-      }
+      final TexturesCategory textureCategory = new TexturesCategory(category);
+      final CatalogTexture texture = new CatalogTexture(name, image, width, height, true);
+      getTexturesCatalog().add(textureCategory, texture);
     }
   }  
 
@@ -583,8 +605,8 @@ public class FileUserPreferences extends UserPreferences {
   @Override
   public void write() throws RecorderException {
     Preferences preferences = getPreferences();
-    writeFurnitureCatalog(preferences);
-    writeTexturesCatalog(preferences);
+    writeModifiableFurnitureCatalog(preferences);
+    writeModifiableTexturesCatalog(preferences);
 
     // Write other preferences 
     preferences.put(LANGUAGE, getLanguage());
@@ -655,9 +677,9 @@ public class FileUserPreferences extends UserPreferences {
   }
 
   /**
-   * Writes furniture catalog in <code>preferences</code>.
+   * Writes modifiable furniture in <code>preferences</code>.
    */
-  private void writeFurnitureCatalog(Preferences preferences) throws RecorderException {
+  private void writeModifiableFurnitureCatalog(Preferences preferences) throws RecorderException {
     final Set<URL> furnitureContentURLs = new HashSet<URL>();
     int i = 1;
     for (FurnitureCategory category : getFurnitureCatalog().getCategories()) {
@@ -733,9 +755,9 @@ public class FileUserPreferences extends UserPreferences {
   }
     
   /**
-   * Writes textures catalog in <code>preferences</code>.
+   * Writes modifiable textures catalog in <code>preferences</code>.
    */
-  private void writeTexturesCatalog(Preferences preferences) throws RecorderException {
+  private void writeModifiableTexturesCatalog(Preferences preferences) throws RecorderException {
     final Set<URL> texturesContentURLs = new HashSet<URL>();
     int i = 1;
     for (TexturesCategory category : getTexturesCatalog().getCategories()) {
@@ -1099,7 +1121,7 @@ public class FileUserPreferences extends UserPreferences {
         throw new RecorderException("Can't access to furniture libraries plugin folder");
       }
       copyToLibraryFolder(new File(furnitureLibraryName), furnitureLibrariesPluginFolders [0]);
-      updateFurnitureDefaultCatalog();
+      updateFurnitureDefaultCatalog(this.catalogsLoader, this.updater);
     } catch (IOException ex) {
       throw new RecorderException(
           "Can't write " + furnitureLibraryName +  " in furniture libraries plugin folder", ex);
@@ -1135,7 +1157,7 @@ public class FileUserPreferences extends UserPreferences {
         throw new RecorderException("Can't access to textures libraries plugin folder");
       }
       copyToLibraryFolder(new File(texturesLibraryName), texturesLibrariesPluginFolders [0]);
-      updateTexturesDefaultCatalog();
+      updateTexturesDefaultCatalog(this.catalogsLoader, this.updater);
     } catch (IOException ex) {
       throw new RecorderException(
           "Can't write " + texturesLibraryName +  " in textures libraries plugin folder", ex);
