@@ -167,6 +167,7 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
       super.setWidth((float)unrotatedBoundingRectangle.getWidth());
       super.setDepth((float)unrotatedBoundingRectangle.getHeight());
       super.setHeight(height - elevation);
+      super.setModelMirrored(modelMirrored);
     } else {
       this.fixedWidth = (float)unrotatedBoundingRectangle.getWidth();
       this.fixedDepth = (float)unrotatedBoundingRectangle.getHeight();
@@ -180,13 +181,14 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     setDescription(null);
     setMovable(movable);
     setVisible(visible);
-    super.setColor(null);
-    super.setTexture(null);
+    if (this.texturable) {
+      super.setColor(null);
+      super.setTexture(null);
+    }
     super.setX((float)center.getX());
     super.setY((float)center.getY());
     super.setAngle(firstPiece.getAngle());
     super.setElevation(elevation);
-    super.setModelMirrored(modelMirrored);
   }
 
   /**
