@@ -529,16 +529,19 @@ public class SwingTools {
       Executors.newSingleThreadExecutor().execute(new Runnable() {
           public void run() {
             try {
+              Thread.sleep(500);
               while (splashScreenWindow.isVisible()) {
-                Thread.sleep(500);
-                // If an other frame is created, dispose splash window
+                // If an other frame is showing, dispose splash window
                 EventQueue.invokeLater(new Runnable() {
                     public void run() {
-                      if (Frame.getFrames().length > 1) {
-                        splashScreenWindow.dispose();
+                      for (Frame frame : Frame.getFrames()) {
+                        if (frame.isShowing()) {
+                          splashScreenWindow.dispose();
+                        }
                       }
                     }
                   });
+                Thread.sleep(300);
               }
             } catch (InterruptedException ex) {
               EventQueue.invokeLater(new Runnable() {
@@ -729,13 +732,13 @@ public class SwingTools {
   /**
    * Returns the children of a component of the given class.
    */
-  public static <T extends JComponent> List<T> findChildren(JComponent parent, Class<T> childrenClass) {
+  public static <T extends Component> List<T> findChildren(JComponent parent, Class<T> childrenClass) {
     List<T> children = new ArrayList<T>();
     findChildren(parent, childrenClass, children); 
     return children;
   }
 
-  private static <T extends JComponent> void findChildren(JComponent parent, Class<T> childrenClass, List<T> children) {
+  private static <T extends Component> void findChildren(JComponent parent, Class<T> childrenClass, List<T> children) {
     for (int i = 0; i < parent.getComponentCount(); i++) {
       Component child = parent.getComponent(i);
       if (childrenClass.isInstance(child)) {
